@@ -25,9 +25,10 @@
 
 import os
 import json
+import copy
 from pathlib import Path
 from .repos import Repo
-from .includehandler import IncludeHandler, IncludeException
+from .includehandler import IncludeHandler
 from .kasusererror import ArtifactNotFoundError
 from .configschema import CONFIGSCHEMA
 
@@ -45,32 +46,25 @@ class Config:
         self._override_target = target
         self._override_task = task
         self._build_dir = ctx.build_dir
-        self._config = {}
+        self.__config = {}
         if not filename:
             filename = os.path.join(ctx.kas_work_dir, CONFIG_YAML_FILE)
 
         self.filenames = [os.path.abspath(configfile)
                           for configfile in filename.split(':')]
-        top_repo_path = Repo.get_root_path(
-            os.path.dirname(self.filenames[0]))
-
-        repo_paths = [Repo.get_root_path(os.path.dirname(configfile),
-                                         fallback=False)
-                      for configfile in self.filenames]
-
-        if len(set(repo_paths)) > 1:
-            raise IncludeException('All concatenated config files must '
-                                   'belong to the same repository or all '
-                                   'must be outside of versioning control')
 
         update = ctx.args.update if hasattr(ctx.args, 'update') else False
 
-        self.handler = IncludeHandler(self.filenames,
-                                      top_repo_path,
-                                      not update)
-        self.repo_dict = self._get_repo_dict()
+        self.handler = IncludeHandler(self.filenames, not update)
+        self.repo_dict = {}
         self.repo_cfg_hashes = {}
 
+    @property
+    def _config(self):
+        if not self.__config:
+            raise RuntimeError("Config has not been imported yet.")
+        return self.__config
+
     def get_build_system(self):
         """
             Returns the pre-selected build system
@@ -79,18 +73,33 @@ class Config:
 
     def find_missing_repos(self, repo_paths={}):
         """
-            Returns repos that are in config but not on disk
+            Returns repos that are in config but not on disk and updates
+            the internal config dictionary.
         """
-        (self._config, missing_repo_names) = \
+        (self.__config, missing_repo_names) = \
             self.handler.get_config(repos=repo_paths)
 
         return missing_repo_names
 
-    def get_config(self):
+    def get_config(self, remove_includes=False, apply_overrides=False):
         """
-            Returns the config dict.
+            Returns a copy of the config dict
         """
-        return self._config
+        config = copy.deepcopy(self._config)
+        if remove_includes and 'includes' in config['header']:
+            del config['header']['includes']
+        if apply_overrides:
+            overrides = config.get('overrides', {})
+            if overrides:
+                config.update(overrides)
+                del config['overrides']
+        return config
+
+    def get_lockfiles(self):
+        """
+            Returns the list of in-use lockfiles.
+        """
+        return self.handler.get_lockfiles()
 
     def get_repos_config(self):
         """
@@ -244,3 +253,14 @@ class Config:
             foundfiles.extend([(name, f) for f in files])
         return [(n, f.relative_to(self._build_dir))
                 for n, f in foundfiles]
+
+    def get_signers_config(self, keytype=None):
+        """
+            Returns the keys from the configuration
+        """
+        signers = self._config.get('signers', {})
+        if not keytype:
+            return signers
+        else:
+            return {k: v for k, v in signers.items()
+                    if v.get('type', 'gpg') == keytype}
