@@ -27,18 +27,20 @@ import tempfile
 import logging
 import shutil
 import os
-import sys
 import pprint
 import configparser
 import json
 import base64
+from pathlib import Path
 from git.config import GitConfigParser
 from .libkas import (ssh_cleanup_agent, ssh_setup_agent, ssh_no_host_key_check,
                      get_build_environ, repos_fetch, repos_apply_patches)
 from .context import ManagedEnvironment as ME
 from .context import get_context
 from .includehandler import IncludeException
-from .kasusererror import ArgsCombinationError
+from .kasusererror import EnvSetButNotFoundError, ArgsCombinationError
+from .keyhandler import GPGKeyHandler, SSHKeyHandler
+from .repos import RepoRefError
 
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2017-2018'
@@ -79,6 +81,7 @@ class Macro:
                 repo_loop,
                 FinishSetupRepos(),
                 ReposCheckout(),
+                ReposCheckSignatures(),
                 ReposApplyPatches(),
                 SetupEnviron(),
                 WriteBBConfig(),
@@ -180,6 +183,7 @@ class SetupHome(Command):
         'AWS_SHARED_CREDENTIALS_FILE',
         'AWS_WEB_IDENTITY_TOKEN_FILE',
         'NETRC_FILE',
+        'NPMRC_FILE',
         'REGISTRY_AUTH_FILE',
     ]
 
@@ -193,6 +197,20 @@ class SetupHome(Command):
     def __str__(self):
         return 'setup_home'
 
+    @staticmethod
+    def _path_from_env(name):
+        """
+        Returns the path a env var points to (if set). If the path does not
+        exist, raises an EnvSetButNotFoundError.
+        """
+        rawpath = os.environ.get(name, None)
+        if not rawpath:
+            return None
+        path = Path(rawpath)
+        if not path.exists():
+            raise EnvSetButNotFoundError(name, path)
+        return path
+
     @staticmethod
     def _ssh_config_present():
         """
@@ -209,9 +227,9 @@ class SetupHome(Command):
         return False
 
     def _setup_netrc(self):
-        if os.environ.get('NETRC_FILE', False):
-            shutil.copy(os.environ['NETRC_FILE'],
-                        self.tmpdirname + "/.netrc")
+        netrc_file = self._path_from_env('NETRC_FILE')
+        if netrc_file:
+            shutil.copy(netrc_file, self.tmpdirname + "/.netrc")
         if os.environ.get('CI_SERVER_HOST', False) \
                 and os.environ.get('CI_JOB_TOKEN', False):
             with open(self.tmpdirname + '/.netrc', 'a') as fds:
@@ -219,10 +237,17 @@ class SetupHome(Command):
                           'login gitlab-ci-token\n'
                           'password ' + os.environ['CI_JOB_TOKEN'] + '\n')
 
+    def _setup_npmrc(self):
+        npmrc_file = self._path_from_env('NPMRC_FILE')
+        if not npmrc_file:
+            return
+        shutil.copy(npmrc_file, self.tmpdirname + "/.npmrc")
+
     def _setup_registry_auth(self):
         os.makedirs(self.tmpdirname + "/.docker")
-        if os.environ.get('REGISTRY_AUTH_FILE', False):
-            shutil.copy(os.environ['REGISTRY_AUTH_FILE'],
+        reg_auth_file = self._path_from_env('REGISTRY_AUTH_FILE')
+        if reg_auth_file:
+            shutil.copy(reg_auth_file,
                         self.tmpdirname + "/.docker/config.json")
         elif not os.path.exists(self.tmpdirname + '/.docker/config.json'):
             with open(self.tmpdirname + '/.docker/config.json', 'w') as fds:
@@ -248,15 +273,17 @@ class SetupHome(Command):
         conf_file = aws_dir + "/config"
         shared_creds_file = aws_dir + "/credentials"
         os.makedirs(aws_dir)
-        if os.environ.get('AWS_CONFIG_FILE') \
-                and os.environ.get('AWS_SHARED_CREDENTIALS_FILE'):
-            shutil.copy(os.environ['AWS_CONFIG_FILE'], conf_file)
-            shutil.copy(os.environ['AWS_SHARED_CREDENTIALS_FILE'],
-                        shared_creds_file)
+        aws_conf_file = self._path_from_env('AWS_CONFIG_FILE')
+        aws_shared_creds_file = \
+            self._path_from_env('AWS_SHARED_CREDENTIALS_FILE')
+        if aws_conf_file and aws_shared_creds_file:
+            shutil.copy(aws_conf_file, conf_file)
+            shutil.copy(aws_shared_creds_file, shared_creds_file)
 
         # OAuth 2.0 workflow credentials
-        if os.environ.get('AWS_WEB_IDENTITY_TOKEN_FILE') \
-                and os.environ.get('AWS_ROLE_ARN'):
+        aws_web_identity_token_file = \
+            self._path_from_env('AWS_WEB_IDENTITY_TOKEN_FILE')
+        if aws_web_identity_token_file and os.environ.get('AWS_ROLE_ARN'):
             webid_token_file = aws_dir + '/web_identity_token'
             config = configparser.ConfigParser()
             if os.path.exists(conf_file):
@@ -267,8 +294,7 @@ class SetupHome(Command):
             config['default']['web_identity_token_file'] = webid_token_file
             with open(aws_dir + '/config', 'w') as fds:
                 config.write(fds)
-            shutil.copy(os.environ['AWS_WEB_IDENTITY_TOKEN_FILE'],
-                        webid_token_file)
+            shutil.copy(aws_web_identity_token_file, webid_token_file)
 
     @staticmethod
     def _setup_gitlab_ci_ssh_rewrite(config):
@@ -292,15 +318,17 @@ class SetupHome(Command):
                              f'ssh://git@{ci_ssh_host}:{ci_ssh_port}/')
 
     def _setup_gitconfig(self):
-        gitconfig_host = os.environ.get('GITCONFIG_FILE', False)
+        gitconfig_host = self._path_from_env('GITCONFIG_FILE')
         gitconfig_kas = self.tmpdirname + '/.gitconfig'
 
         # when running in a externally managed environment,
         # always try to read the gitconfig
         if not gitconfig_host and get_context().managed_env:
-            gitconfig_host = os.path.expanduser('~/.gitconfig')
+            gitconfig_host = Path('~/.gitconfig').expanduser()
+            if not gitconfig_host.exists():
+                gitconfig_host = None
 
-        if gitconfig_host and os.path.exists(gitconfig_host):
+        if gitconfig_host:
             shutil.copy(gitconfig_host, gitconfig_kas)
 
         with GitConfigParser(gitconfig_kas, read_only=False) as config:
@@ -311,13 +339,19 @@ class SetupHome(Command):
                 if os.environ.get('GIT_CREDENTIAL_USEHTTPPATH', False):
                     config['credential']['useHttpPath'] = \
                         os.environ.get('GIT_CREDENTIAL_USEHTTPPATH')
-            # in GitLab CI, add ssh -> https rewrites if no config is present
-            ci_server = os.environ.get('CI_SERVER_HOST', None)
-            if get_context().managed_env == ME.GITLAB_CI and ci_server and \
-                    not self._ssh_config_present() and \
-                    not os.path.exists(gitconfig_host):
-                logging.debug('Adding GitLab CI ssh -> https rewrites')
-                self._setup_gitlab_ci_ssh_rewrite(config)
+
+            if get_context().managed_env == ME.GITLAB_CI and \
+                    not gitconfig_host:
+                ci_project_dir = self._path_from_env('CI_PROJECT_DIR')
+                if ci_project_dir:
+                    logging.debug('Adding git safe.directory %s',
+                                  ci_project_dir)
+                    config.add_value('safe', 'directory', str(ci_project_dir))
+
+                ci_server = os.environ.get('CI_SERVER_HOST', None)
+                if ci_server and not self._ssh_config_present():
+                    logging.debug('Adding GitLab CI ssh -> https rewrites')
+                    self._setup_gitlab_ci_ssh_rewrite(config)
             config.write()
 
     def execute(self, ctx):
@@ -326,6 +360,7 @@ class SetupHome(Command):
             logging.info(f'Running on {managed_env}')
         def_umask = os.umask(0o077)
         self._setup_netrc()
+        self._setup_npmrc()
         self._setup_registry_auth()
         self._setup_gitconfig()
         self._setup_aws_creds()
@@ -343,7 +378,6 @@ class SetupDir(Command):
         return 'setup_dir'
 
     def execute(self, ctx):
-        os.chdir(ctx.kas_work_dir)
         if not os.path.exists(ctx.build_dir):
             os.mkdir(ctx.build_dir)
 
@@ -550,11 +584,7 @@ class FinishSetupRepos(Command):
         # now fetch everything with complete config
         repos_fetch(ctx.config.get_repos())
 
-        if sys.version_info < (3, 8):
-            config_str = pprint.pformat(ctx.config.get_config())
-        else:
-            config_str = pprint.pformat(ctx.config.get_config(),
-                                        sort_dicts=False)
+        config_str = pprint.pformat(ctx.config.get_config(), sort_dicts=False)
         logging.debug('Configuration from config file:\n%s', config_str)
 
 
@@ -569,3 +599,53 @@ class ReposCheckout(Command):
     def execute(self, ctx):
         for repo in ctx.config.get_repos():
             repo.checkout()
+
+
+class ReposCheckSignatures(Command):
+    """
+        Imports the keys defined in the configuration and checks the
+        signatures of the repositories.
+    """
+
+    def __str__(self):
+        return 'repos_check_signatures'
+
+    def execute(self, ctx):
+        self._import_keys(ctx)
+        self._check_signatures(ctx)
+
+    def _import_keys(self, ctx):
+        handler_cfg = {
+            'gpg': (GPGKeyHandler,
+                    Path(ctx.kas_work_dir) / '.kas_gnupg'),
+            'ssh': (SSHKeyHandler,
+                    Path(ctx.kas_work_dir) / '.kas_ssh-handler'),
+        }
+        for name, (handler_cls, dir) in handler_cfg.items():
+            signers_cfg = ctx.config.get_signers_config(name)
+            if not signers_cfg:
+                continue
+            dir.mkdir(exist_ok=True)
+            dir.chmod(0o700)
+            ctx.managed_paths.add(dir)
+            ctx.keyhandler[name] = handler_cls(dir, signers_cfg, ctx.config)
+
+        for keyhandler in ctx.keyhandler.values():
+            ctx.environ.update(keyhandler.env)
+
+    def _check_signatures(self, ctx):
+        for repo in ctx.config.get_repos():
+            if not repo.signed:
+                continue
+            valid, keyid = repo.check_signature()
+            keyhandler = ctx.keyhandler[repo.signers_type]
+            info = keyhandler.get_key_repr(keyid) if keyid else 'No info'
+            if valid:
+                logging.info(f'Repository {repo.name} signature valid: {info}')
+                continue
+            elif keyid:
+                raise RepoRefError(f'Repository {repo.name} is not signed '
+                                   f'with a trusted key: {info}')
+
+            raise RepoRefError(f'Repository {repo.name} is not signed '
+                               'with a trusted key.')
