@@ -109,7 +109,15 @@ async def _read_stream(stream, callback):
             break
 
 
-async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=False):
+def _filter_stderr(capture_stderr, ret, out, err=None):
+    if capture_stderr:
+        return (ret, out, err or '')
+    else:
+        return (ret, out)
+
+
+async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=False,
+                        capture_stderr=False):
     """
         Run a command asynchronously.
     """
@@ -120,12 +128,8 @@ async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=False):
 
     logo = LogOutput(liveupdate)
 
-    try:
-        orig_fd = signal.set_wakeup_fd(-1, warn_on_full_buffer=False)
-        signal.set_wakeup_fd(orig_fd, warn_on_full_buffer=False)
-    except TypeError:
-        # Python < 3.7 - we tried our best
-        pass
+    orig_fd = signal.set_wakeup_fd(-1, warn_on_full_buffer=False)
+    signal.set_wakeup_fd(orig_fd, warn_on_full_buffer=False)
 
     try:
         process = await asyncio.create_subprocess_exec(
@@ -138,11 +142,11 @@ async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=False):
     except FileNotFoundError as ex:
         if fail:
             raise ex
-        return (errno.ENOENT, str(ex))
+        return _filter_stderr(capture_stderr, errno.ENOENT, str(ex))
     except PermissionError as ex:
         if fail:
             raise ex
-        return (errno.EPERM, str(ex))
+        return _filter_stderr(capture_stderr, errno.EPERM, str(ex))
 
     # Process termination is a complicated thing. We need to ensure that
     # the event-loop ThreadedChildWatcher thread fires before the loop is
@@ -174,10 +178,11 @@ async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=False):
         logging.error(msg)
         raise CommandExecError(cmd, ret)
 
-    return (ret, ''.join(logo.stdout))
+    return _filter_stderr(capture_stderr, ret,
+                          ''.join(logo.stdout), ''.join(logo.stderr))
 
 
-def run_cmd(cmd, cwd, env=None, fail=True):
+def run_cmd(cmd, cwd, env=None, fail=True, capture_stderr=False):
     """
         Runs a command synchronously.
     """
@@ -192,12 +197,14 @@ def run_cmd(cmd, cwd, env=None, fail=True):
     except FileNotFoundError as ex:
         if fail:
             raise ex
-        return (errno.ENOENT, str(ex))
+        return _filter_stderr(capture_stderr, errno.ENOENT, str(ex))
     except PermissionError as ex:
         if fail:
             raise ex
-        return (errno.EPERM, str(ex))
-    return (ret.returncode, ret.stdout.decode('utf-8'))
+        return _filter_stderr(capture_stderr, errno.EPERM, str(ex))
+    return _filter_stderr(capture_stderr, ret.returncode,
+                          ret.stdout.decode('utf-8'),
+                          ret.stderr.decode('utf-8'))
 
 
 def find_program(paths, name):
@@ -439,10 +446,6 @@ def setup_parser_common_args(parser):
     from kas.libcmds import Macro
 
     setup_cmds = [str(s) for (s, _) in Macro().setup_commands]
-    parser.add_argument('config',
-                        help='Config file(s), separated by colon. Using '
-                        '.config.yaml in KAS_WORK_DIR if none is specified.',
-                        nargs='?')
     parser.add_argument('--skip',
                         help='Skip build steps. To skip more than one step, '
                         'use this argument multiple times.',
@@ -458,6 +461,14 @@ def setup_parser_common_args(parser):
                         'branch even if it is already checked out locally')
 
 
+def setup_parser_config_arg(parser):
+    parser.add_argument('config',
+                        help='Config file(s), separated by colon. Using '
+                        '.config.yaml in KAS_WORK_DIR if none is '
+                        'specified.',
+                        nargs='?')
+
+
 def setup_parser_preserve_env_arg(parser):
     parser.add_argument('-E', '--preserve-env',
                         help='Keep current user environment block',
