@@ -62,7 +62,7 @@
       * ``KAS_REPO_REFSPEC``: The refspec which was checked out for this
         repository, or an empty string if no refspec was given in the config
         file. This variable is obsolete and will be removed when support for
-        respec keys is removed as well. Migrate your repos to commit/branch
+        refspec keys is removed as well. Migrate your repos to commit/branch
         and use the related variables instead.
 """
 
@@ -72,7 +72,7 @@ import subprocess
 from kas.context import create_global_context
 from kas.config import Config
 from kas.libcmds import Macro, Command, SetupHome
-from kas.libkas import setup_parser_common_args
+from kas.libkas import setup_parser_common_args, setup_parser_config_arg
 from kas.libkas import setup_parser_keep_config_unchanged_arg
 from kas.libkas import setup_parser_preserve_env_arg
 from kas.libkas import run_handle_preserve_env_arg
@@ -91,6 +91,7 @@ class ForAllRepos:
     @classmethod
     def setup_parser(cls, parser):
         setup_parser_common_args(parser)
+        setup_parser_config_arg(parser)
         setup_parser_preserve_env_arg(parser)
         setup_parser_keep_config_unchanged_arg(parser)
         parser.add_argument('command',
