@@ -25,7 +25,6 @@
 
 import re
 import os
-import sys
 import linecache
 import logging
 import shutil
@@ -35,11 +34,8 @@ from tempfile import TemporaryDirectory
 from .context import get_context
 from .libkas import run_cmd_async, run_cmd
 from .kasusererror import KasUserError
-
-if sys.version_info < (3, 8):
-    from cached_property import cached_property
-else:
-    from functools import cached_property
+from functools import cached_property
+from git import Repo as GitPythonRepo
 
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2017-2018'
@@ -77,6 +73,14 @@ class PatchApplyError(KasUserError):
     """
     The provided patch file could not be applied
     """
+    def __init__(self, msg, cmd=None, out=None, err=None):
+        if cmd:
+            msg += '\nvcs command: ' + ' '.join(cmd)
+        if out and out.strip():
+            msg += f'\nvcs output:\n{out.strip()}'
+        if err and err.strip():
+            msg += f'\nvcs error:\n{err.strip()}'
+        super().__init__(msg)
 
 
 class Repo:
@@ -85,7 +89,7 @@ class Repo:
     """
 
     def __init__(self, name, url, path, commit, tag, branch, refspec, layers,
-                 patches, disable_operations):
+                 patches, signers, disable_operations):
         self.name = name
         self.url = url
         self.path = path
@@ -95,6 +99,7 @@ class Repo:
         self.refspec = refspec
         self._layers = layers
         self._patches = patches
+        self.allowed_signers = signers
         self.operations_disabled = disable_operations
 
         if not self.url:
@@ -157,6 +162,41 @@ class Repo:
                               cwd=self.path, fail=False)
         return bool(output)
 
+    @cached_property
+    def signers_type(self):
+        if self.allowed_signers is None:
+            return 'gpg'
+        signers = get_context().config.get_signers_config()
+        try:
+            ktypes = [signers[k].get('type', 'gpg')
+                      for k in self.allowed_signers]
+        except KeyError as e:
+            raise KasUserError(f'Repository {self.name}: '
+                               f'Allowed signer "{e}" not found in config')
+        if len(set(ktypes)) > 1:
+            raise KasUserError(f'Repository {self.name}: '
+                               'Mixed signer types are not supported')
+        return ktypes[0]
+
+    @property
+    def keyhandler(self):
+        if not self.signed:
+            return None
+        return get_context().keyhandler[self.signers_type]
+
+    def check_signature(self):
+        self.keyhandler.prepare_validation(self)
+        (ret, _, err) = run_cmd(self.is_signed_cmd(),
+                                cwd=self.path, fail=False, capture_stderr=True)
+        logging.debug('Signature verification output (%d):\n%s', ret, err)
+        if ret != 0:
+            return (False, None)
+        return self.keyhandler.validate_allowed_signer(self, err)
+
+    @cached_property
+    def signed(self):
+        return self.allowed_signers is not None
+
     @staticmethod
     def get_type():
         """
@@ -185,10 +225,17 @@ class Repo:
     def factory(name, repo_config, repo_defaults, repo_fallback_path,
                 repo_overrides={}):
         """
-            Returns a Repo instance depending on params.
+            Returns a Repo instance depending on parameters.
             This factory function is referential transparent.
         """
         layers_dict = repo_config.get('layers', {'': None})
+        # only bool(false) will be a valid value to disable a layer
+        for lname, prop in layers_dict.items():
+            if not (prop is None or prop == "disabled"):
+                logging.warning('Use of deprecated value "%s" for repo '
+                                '"%s", layer "%s". Replace with "disabled".',
+                                prop, name, lname)
+
         layers = list(filter(lambda x, laydict=layers_dict:
                              str(laydict[x]).lower() not in
                              ['disabled', 'excluded', 'n', 'no', '0', 'false'],
@@ -246,6 +293,11 @@ class Repo:
                                 '"%s" is unsafe as tags are mutable.', name)
                 Repo.__no_commit_tag_warned__.append(name)
         path = repo_config.get('path', None)
+        signed = repo_config.get('signed', False)
+        signers = repo_config.get('allowed_signers', None) if signed else None
+        if signed and not signers:
+            raise KasUserError(f'Repository "{name}" is signed but no allowed '
+                               'signers specified.')
         disable_operations = False
 
         if path is None:
@@ -268,13 +320,13 @@ class Repo:
                     f'{commit} is not a full-length hash for repo '
                     f'"{name}". This will be an error in future versions.')
             return GitRepo(name, url, path, commit, tag, branch, refspec,
-                           layers, patches, disable_operations)
+                           layers, patches, signers, disable_operations)
         if repo_type == 'hg':
             if not shutil.which('hg'):
                 raise UnsupportedRepoTypeError(
                     'hg is required for Mercurial repositories')
             return MercurialRepo(name, url, path, commit, tag, branch, refspec,
-                                 layers, patches, disable_operations)
+                                 layers, patches, signers, disable_operations)
         raise UnsupportedRepoTypeError(f'Repo type "{repo_type}" '
                                        'not supported.')
 
@@ -327,9 +379,6 @@ class RepoImpl(Repo):
                 logging.debug('Created repo ref for %s', self.qualified_name)
                 try:
                     os.rename(tmpdir, sdir)
-                    if sys.version_info < (3, 8):
-                        # recreate dir so cleanup handler can delete it
-                        os.makedirs(tmpdir, exist_ok=True)
                 except OSError:
                     logging.debug('repo %s already cloned by other instance',
                                   self.qualified_name)
@@ -508,24 +557,24 @@ class RepoImpl(Repo):
 
         for (path, patch_id) in my_patches:
             cmd = self.apply_patches_file_cmd(path)
-            (retc, output) = await run_cmd_async(
-                cmd, cwd=self.path, fail=False)
+            (retc, out, err) = await run_cmd_async(
+                cmd, cwd=self.path, fail=False, capture_stderr=True)
             if retc:
                 raise PatchApplyError(
-                    'Could not apply patch. Please fix repos and patches. '
-                    f'(patch path: {path}, repo: {self.name}, patch '
-                    f'entry: {patch_id}, vcs output: {output})')
+                    'Could not apply patch. Please fix repos and patches:\n'
+                    f'patch path: {path}, repo: {self.name}, patch '
+                    f'entry: {patch_id}', cmd, out, err)
 
             logging.info('Patch applied. '
                          '(patch path: %s, repo: %s, patch entry: %s)',
                          path, self.name, patch_id)
 
             cmd = self.add_cmd()
-            (retc, output) = await run_cmd_async(
-                cmd, cwd=self.path, fail=False)
+            (retc, out, err) = await run_cmd_async(
+                cmd, cwd=self.path, fail=False, capture_stderr=True)
             if retc:
-                raise PatchApplyError('Could not add patched files. repo: '
-                                      f'{self.name}, vcs output: {output})')
+                raise PatchApplyError('Could not add patched files: repo: '
+                                      f'{self.name}', cmd, out, err)
 
             timestamp = self.get_patch_timestamp(path)
             if not timestamp:
@@ -537,11 +586,11 @@ class RepoImpl(Repo):
             msg = f'kas: {patch_id}\n\npatch {path} applied by kas'
             cmd = self.commit_cmd(env, 'kas <kas@example.com>', msg,
                                   timestamp)
-            (retc, output) = await run_cmd_async(
-                cmd, cwd=self.path, env=env, fail=False)
+            (retc, out, err) = await run_cmd_async(
+                cmd, cwd=self.path, env=env, fail=False, capture_stderr=True)
             if retc:
                 raise PatchApplyError('Could not commit patch changes. repo: '
-                                      f'{self.name}, vcs output: {output})')
+                                      f'{self.name}', cmd, out, err)
 
         return 0
 
@@ -586,7 +635,14 @@ class GitRepo(RepoImpl):
                                 f'refspec on repository "{self.name}". '
                                 'Performing full clone.')
             else:
-                cmd.extend(['--depth', str(depth)])
+                if createref:
+                    # this is not a user-error, as the clone of the work repo
+                    # can still be shallow.
+                    logging.debug('Shallow cloning is not supported for '
+                                  f'reference repository of "{self.name}". '
+                                  'Performing full clone.')
+                else:
+                    cmd.extend(['--depth', str(depth)])
                 if self.branch:
                     cmd.extend(['--branch',
                                 self.remove_ref_prefix(self.branch)])
@@ -636,6 +692,13 @@ class GitRepo(RepoImpl):
     def is_dirty_cmd(self):
         return ['git', 'diff', '--stat']
 
+    def is_signed_cmd(self):
+        if self.tag:
+            return ['git', 'verify-tag', '--raw', self.tag]
+        else:
+            refspec = self.commit or self.revision
+            return ['git', 'verify-commit', '--raw', refspec]
+
     def resolve_branch_cmd(self):
         refspec = self.remove_ref_prefix(self.branch or self.refspec)
         return ['git', 'rev-parse', '--verify', '-q', f'origin/{refspec}']
@@ -684,6 +747,29 @@ class GitRepo(RepoImpl):
     def contains_path_cmd(self, path):
         return ['git', 'ls-files', '--error-unmatch', path]
 
+    def diff(self, commit1, commit2):
+        if commit1 is None:
+            commit1 = 'HEAD'
+        if commit2 is None:
+            commit2 = 'HEAD'
+        git_repo = GitPythonRepo(self.path)
+        shallow_file = os.path.join(git_repo.git_dir, 'shallow')
+        if os.path.isfile(shallow_file):
+            git_repo.git.fetch(unshallow=True)
+        commits = list(git_repo.iter_commits(
+                       f'{commit1}..{commit2}'))
+        diff_json = {self.name: []}
+        for commit in commits:
+            diff_json[self.name].append({
+                'commit': commit.hexsha,
+                'author': commit.author.name,
+                'email': commit.author.email,
+                'commit_date': commit.committed_datetime.
+                strftime("%Y-%m-%d %H:%M:%S"),
+                'message': commit.message
+            })
+        return diff_json
+
 
 class MercurialRepo(RepoImpl):
     """
@@ -717,6 +803,9 @@ class MercurialRepo(RepoImpl):
         return ['hg', 'status', '--modified', '--added',
                 '--removed', '--deleted']
 
+    def is_signed_cmd(self):
+        raise NotImplementedError()
+
     def resolve_branch_cmd(self):
         if self.branch:
             return ['hg', 'identify', '--id', '-r',
@@ -767,3 +856,6 @@ class MercurialRepo(RepoImpl):
 
     def contains_path_cmd(self, path):
         return ['hg', 'files', path]
+
+    def diff(self, commit1, commit2):
+        raise NotImplementedError("Unsupported diff for MercurialRepo")
